#!/bin/bash

# Set some defaults
INSTALLFILE="install_files_list.nsh"
UNINSTALLFILE="uninstall_files_list.nsh"
LICENSEFILE="license_page.nsh"
MAKENSISPATH="C:\Program Files\NSIS\makensis.exe"

# show usage when invoked without args
if [ "$#" -eq "0" ]
then
  cat << EOF
Helper script to build a proper Windows installer out of a
Pd build tree.

Usage:
./build.sh --pd-win-dir <path_to_pd_build>
EOF
  exit 1
fi

# Parse commandline
while [ $# -gt 0 ]
do
  case "$1" in
    --pd-win-dir) PDWINDIR=$2; shift;;
    -*) echo "error - unrecognized option '$1'" 1>&2
        exit 1;;
    *)  break;;
  esac
  shift
done

# Check validity of specified PDWINDIR
if ! [ -d "$PDWINDIR" ]
then
  echo "$PDWINDIR is not a directory"
  exit 1
elif ! [ -f "$PDWINDIR/bin/pd.exe" ]
then
  echo "Could not find bin/pd.exe. Is this a Windows build?"
  exit 1
fi

#### WRITE INSTALL LIST #########################################
find "$PDWINDIR" \
  -mindepth 0 \
  -type d \
  -printf 'SetOutPath "$INSTDIR/%P"\n' \
  -exec \
    find {} \
    -maxdepth 1 \
    -type f \
    -printf 'File "%p"\n' \; \
    -not -name "*.o" \
  | sed 's|/|\\|g' \
  > $INSTALLFILE

#### WRITE UNINSTALL FILE LIST ###################################
# clear
> $UNINSTALLFILE

# delete files
find "$PDWINDIR" -type f -printf 'Delete "$INSTDIR/%P"\n' \
  -not -name "*.o" \
  | sed 's|/|\\|g' \
  >> $UNINSTALLFILE

# delete directories
# tac is used to reverse order so that dirs are removed in depth-first order
find "$PDWINDIR" -type d -printf 'RMDir "$INSTDIR/%P"\n' \
  | tac \
  | sed 's|/|\\|g' \
  >> $UNINSTALLFILE

#### WRITE LICENSE INCLUDE ######################################
echo "!insertmacro MUI_PAGE_LICENSE \"$PDWINDIR/LICENSE.txt\"" \
  > $LICENSEFILE

#### BUILD WINDOWS INSTALLER ####################################
# check if we have wine
if ! which wine
then
  echo "wine is not installed. Exiting."
  exit 1
fi

# check if we have nsis compiler
nsis_version=$(wine "$MAKENSISPATH" /version)
nsis_exit=$?
case $nsis_exit in
  0) echo "Found NSIS version $nsis_version"
     ;;
  1) echo "makensis returned error. What's the problem?"
     exit 1
     ;;
  2) echo "NSIS is not found. We need NSIS from:"
     echo "http://nsis.sourceforge.net/Download"
     exit 1
     ;;
  *) echo "Unkown error"
     exit 1
     ;;
esac

# run the build
if wine "$MAKENSISPATH" pd.nsi
then
  echo "Build successfull"
else
  echo "Some error occured during compilation"
  exit 1
fi
exit 0
